import pytest
from time import sleep

from pycdm import PyCDM
from pycdm.structs import BsdaSettings, FramingType, EbiSettings
import cdm_bindings

"""
XXX: this integration tests only works with a single BCG, not an entire baffle. If both halves are present, the test is not valid.
"""

def test_cfg():
    s = PyCDM("127.0.0.1")
    s._cfg.get_cdm_addr()
    s.close()

def test_cdm_commands(tmpdir):
    s = PyCDM("127.0.0.1")
    s.cdm.get_info()
    s.cdm.get_config()
    s.cdm.get_warnings()
    assert s.cdm.get_sm_state() == cdm_bindings.SM_STATE.INITIALIZED
    s.cdm.power_off("I_KNOW_WHAT_I_AM_DOING")
    s.cdm.power_on("I_KNOW_WHAT_I_AM_DOING")
    with pytest.raises(Exception):
        s.cdm.power_off()
    with pytest.raises(Exception):
        s.cdm.power_on()
    s.cdm.download_file(2025, 4, 1, 12, "PS.VLT", "{}/file".format(tmpdir)) # XXX fix the hardocded file name or ensure to create it
    s.close()

def test_ebi_commands():
    s = PyCDM("127.0.0.1")
    s.ebi.connect()
    try:
        r = s.bcg.get_state()
        loc = "right" if r["right"] else "left"
        s.ebi.echo(loc[0])
        prev = s.bcg.get_info()
        s.ebi.hard_reset()
        sleep(5)
        s.ebi.connect()
        aft = s.bcg.get_info()
        assert prev[loc]["ebi"]["uptime"] > aft[loc]["ebi"]["uptime"]
        for i, k  in zip(prev[loc]["bsda"], aft[loc]["bsda"]):
            # BSDA are not reset, so the uptime should have increased
            assert i["uptime"] < k["uptime"]
    finally:
        s.ebi.disconnect()
        s.close()

def test_bsda_commands(tmpdir):
    s = PyCDM("127.0.0.1")
    s.ebi.connect()
    try:
        r = s.bcg.get_state()
        loc = "right" if r["right"] else "left"
        for i in range(3):
            s.bsda.echo(loc[0], i)
        prev = s.bcg.get_info()
        s.bsda.hard_reset()
        sleep(5)
        aft = s.bcg.get_info()
        # EBI is not reset
        assert prev[loc]["ebi"]["uptime"] < aft[loc]["ebi"]["uptime"]
        for i, k  in zip(prev[loc]["bsda"], aft[loc]["bsda"]):
            assert i["uptime"] > k["uptime"]

    finally:
        s.ebi.disconnect()
        s.close()


def test_bcg_commands(tmpdir):
    s = PyCDM("127.0.0.1")
    s.ebi.connect()
    try:
        s.bcg.get_running_partition()
        s.bcg.get_voltage()
        s.bcg.get_adc_map()
        s.bcg.get_config()
        s.bcg.versions()
        r = s.bcg.get_state()
        loc = "right" if r["right"] else "left"
        prev = s.bcg.get_info()
        if loc == "right":
            s.bcg.dump_log(["R", "R0", "R1", "R2"])
        else:
            s.bcg.dump_log(["L", "L0", "L1", "L2"])

        s.bcg.hard_reset()
        sleep(5)
        s.ebi.connect()
        aft = s.bcg.get_info()
        assert prev[loc]["ebi"]["uptime"] > aft[loc]["ebi"]["uptime"]
        for i, k  in zip(prev[loc]["bsda"], aft[loc]["bsda"]):
            assert i["uptime"] > k["uptime"]
    finally:
        s.ebi.disconnect()
        s.close()

def test_short_acq(tmpdir):
    bsda = BsdaSettings()
    bsda.framing = FramingType.SUM_SQ
    bsda.frame_duration_us = 1000
    bsda.temperature_period_ms = 1000
    bsda.voltage_period_ms = 12000
    bsda.adc_shadow = [0xf9f0, 0xf9f0]
    bsda.log_uart = True
    bsda.log_level = 3
    bsda.raw_adc = 1
    bsda.raw_channel = 1
    bsda.disabled = False
    bsda.light_enabled = True
    bsda.temperature_enabled = True
    bsda.voltage_enabled = True

    bsda_disabled = BsdaSettings()
    bsda_disabled.disabled = True

    left = EbiSettings()
    left.frame_reader_framing_type = FramingType.SUM_SQ
    left.frame_reader_period_ms = 15
    left.temperature_reader_period_ms = 10000
    left.voltage_reader_period_ms = 12000
    left.raw_board = 0
    left.bsda_settings[0] = bsda
    left.bsda_settings[1] = bsda
    left.bsda_settings[2] = bsda
    left.ntp_period_m = 1
    left.log_uart = True
    left.log_level = 2

    right = EbiSettings()
    right.frame_reader_framing_type = FramingType.SUM_SQ
    right.frame_reader_period_ms = 15
    right.temperature_reader_period_ms = 10000
    right.voltage_reader_period_ms = 12000
    right.raw_board = 1
    right.bsda_settings[0] = bsda
    right.bsda_settings[1] = bsda
    right.bsda_settings[2] = bsda
    right.ntp_period_m = 1
    right.log_uart = True
    right.log_level = 2

    s = PyCDM("127.0.0.1")
    s.ebi.connect()
    try:
        r = s.bcg.get_state()
        # At least one side have to be available for the reading
        if r["right"] is None:
            right.bsda_settings[0] = bsda_disabled
            right.bsda_settings[1] = bsda_disabled
            right.bsda_settings[2] = bsda_disabled
        elif r["left"] is None:
            left.bsda_settings[0] = bsda_disabled
            left.bsda_settings[1] = bsda_disabled
            left.bsda_settings[2] = bsda_disabled

        s.configure(left, right)
        # configure two times to test the other path and also to test the re-configure
        s.configure(left)

        s.start_acq()
        # 2 minutes should be enough to exercise the voltage, ntp, temperature and light readers
        sleep(2 * 60)
        s.stop_acq()
    finally:
        s.ebi.disconnect()
        s.close()



import time
from pycdm import PyCDM
import cdm_bindings
from pycdm.structs import BsdaSettings, FramingType, EbiSettings
import zmq, json

cdm = PyCDM("127.0.0.1")
s = cdm.cdm.get_sm_state()
if s == cdm_bindings.SM_STATE.INITIALIZED:
    cdm.ebi.connect()

while True:
    i = cdm.bcg.get_info()
    if "left" in i:
        l = i["left"]["ebi"]
        uptime = l["uptime"]/1000000/60
        free = l["heap"]["largest_free_block"]
        total = l["heap"]["total_free_bytes"]
        fragmented = (total - free)/1024
        print("L UP: {:.2f}m".format(uptime))
        print("L largest_free: {:.2f}kb, total: {:.2f}kb, fragmented_waste: {:.2f}kb".format(free/1024, total/1024, fragmented))

    if "right" in i:
        r = i["right"]["ebi"]
        uptime = r["uptime"]/1000000/60
        free = r["heap"]["largest_free_block"]
        total = r["heap"]["total_free_bytes"]
        fragmented = (total - free)/1024
        print("R UP: {:.2f}m".format(uptime))
        print("R largest_free: {:.2f}kb, total: {:.2f}kb, fragmented_waste: {:.2f}kb".format(free / 1024, total / 1024, fragmented))

    time.sleep(5)

print("Closing")
cdm.close()



import queue
import logging
from threading import Thread

import zmq.error

from .comm import ZmqAsyncConn

log = logging.getLogger(__name__)

Msg = tuple[str, bytes]


class MsgReg(queue.Queue[Msg]):

    def __init__(self, i: str) -> None:
        super().__init__()
        self._id = i

    def id_match(self, i: str) -> bool:
        return self._id == i[0:len(self._id)]

    def add_msg(self, msg: Msg) -> None:
        self.put(msg)

    def get_msg(self) -> bytes:
        return self.get(block=True)[1]


class DataManager:

    def __init__(self, addr: str) -> None:
        self._cdm_async = ZmqAsyncConn(addr)

        self._thread = Thread(target=self._frame_reader)
        self._thread.start()

        self._msg_reg: list[MsgReg] = []

    def close(self) -> None:
        log.info("Closing DataManager")
        self._cdm_async.disconnect()
        self._thread.join()

    def register(self, s: str) -> MsgReg:
        reg = MsgReg(s)
        self._msg_reg.append(reg)
        self._cdm_async.subscribe(s)
        return reg

    def _frame_reader(self) -> None:
        log.info("FRAME READER STARTED")
        try:
            while True:
                m = self._cdm_async.recv_msg()
                for i in self._msg_reg:
                    if i.id_match(m[0]):
                        i.add_msg(m)
        except zmq.error.ContextTerminated:
            log.warning("Exiting from frame reader")



import json
from pycdm import PyCDM, BsdaSettings, FramingType, EbiSettings

cdm = PyCDM("127.0.0.1")
cdm.ebi.connect()

# To read data only from one side or a specific BSDA use
# DATA.TEMP.R or DATA.TEMP.L (to read only from left or right side)
# or
# DATA.TEMP.RN where N can be 0,1,2
# It also works the same for DATA.FRAME
temperature_msg_queue = cdm.reg_msg("DATA.TEMP")
data_msg_queue = cdm.reg_msg("DATA.FRAME")
# Other values that can be registered are:
#   PS.VLT
#   PS.AMP
#   LAST_COMM
#   DISK


bsda = BsdaSettings()
bsda.framing = FramingType.FULL
bsda.frame_duration_us = 1000
bsda.temperature_period_ms = 90000
#bsda.adc_shadow = [0xf9f, 0xf9f]
bsda.adc_shadow = [0xf9f0, 0xf9f0]
bsda.log_uart = True
bsda.log_level = 3
bsda.raw_adc = 1
bsda.raw_channel = 1
bsda.disabled = False
bsda.light_enabled = False
bsda.temperature_enabled = True

bsda_disabled = BsdaSettings()
bsda_disabled.disabled = True

left = EbiSettings()
left.frame_reader_framing_type = FramingType.FULL
left.frame_reader_period_ms = 15
left.temperature_reader_period_ms = 100
left.raw_board = 0
left.bsda_settings[0] = bsda_disabled
left.bsda_settings[1] = bsda_disabled
left.bsda_settings[2] = bsda_disabled
left.log_uart = True
left.log_level = 3

right = EbiSettings()
right.frame_reader_framing_type = FramingType.FULL
right.frame_reader_period_ms = 15
right.temperature_reader_period_ms = 100
right.raw_board = 1
right.bsda_settings[0] = bsda
right.bsda_settings[1] = bsda_disabled
right.bsda_settings[2] = bsda_disabled
right.log_uart = True
right.log_level = 3

cdm.configure(left, right)

while True:
    j = json.loads(temperature_msg_queue.get_msg())
    print(j)
    #print("Board: {}, I2C: {}, PT100: {}".format(j["board"], j["i2c"], j["pt100"]))

cdm.close()



import base64, zlib
from hashlib import sha256


def encode_from_file(input_file: str) -> tuple[bytes, str]:
    with open(input_file, "rb") as f:
        fw = f.read()

    fw_hash = sha256(fw).hexdigest()
    fw_zip = zlib.compress(fw)
    fw_zip_b64 = base64.b64encode(fw_zip)

    return fw_zip_b64, fw_hash


def decode_to_file(encoded_str: bytes | str, dest_file: str) -> None:
    file_zip = base64.b64decode(encoded_str)
    file = zlib.decompress(file_zip)
    with open(dest_file, "xb") as f:
        f.write(file)


def check_valid_board_identifier(s: str) -> str:
    if len(s) > 2:
        raise Exception("Invalid value: {}", s)

    if s[0] != "R" and s[0] != "L":
        raise Exception("Invalid value: {}", s[0])

    if len(s) == 2:
        if int(s[1]) > 3 or int(s[1]) < 0:
            raise Exception("Invalid value: {}", s[0])

    return s



import signal
from types import FrameType
from typing import Any

import cdm_bindings

from .cfg import Cfg
from .data_mgr import DataManager, MsgReg
from .cmd_mgr import CmdUtils, CdmCmd, BcgCmd, EbisCmd, BsdasCmd
from .structs import EbiSettings

class PyCDM:
    """
    This class exports only the bare minimum commands needed for the normal operation of the system.
    All other commands are exported by the different classes contained by this one.
    """
    def __init__(self, cdm_addr: str = "") -> None:
        self._cfg: Cfg = Cfg()
        if cdm_addr == "":
            cdm_addr = self._cfg.get_cdm_addr()
        self._cmd: CmdUtils = CmdUtils(cdm_addr)
        self._data: DataManager = DataManager(cdm_addr)
        signal.signal(signal.SIGINT, self._handler)

        # Commands targeted to CDM
        self.cdm: CdmCmd = CdmCmd(self._cmd)

        # Commands targeted to one or more BCG(s)
        self.bcg: BcgCmd = BcgCmd(self._cmd)

        # Commands targeted at one or more EBI(s)
        self.ebi: EbisCmd = EbisCmd(self._cmd)

        # Commands targeted at one or more BSDA(s)
        self.bsda: BsdasCmd = BsdasCmd(self._cmd)

    def _handler(self, signum: int, frame: FrameType | None) -> None:
        self.close()

    def close(self) -> None:
        self._data.close()

    def check_comm(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.check)

    def configure(self, baffle_settings: EbiSettings, right_baffle_settings: EbiSettings | None = None) -> dict[str, Any]:
        if right_baffle_settings:
            d = {"left": baffle_settings.to_json(), "right": right_baffle_settings.to_json()}
            return self._cmd.process(cdm_bindings.configure, d)
        else:
            d = {"left": baffle_settings.to_json(), "right": baffle_settings.to_json()}
            return self._cmd.process(cdm_bindings.configure, d)

    def start_acq(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.start)

    def stop_acq(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.stop)

    def read_pt100(self, ebi: str, bsda: int) -> tuple[list[float | int], int, float]:
        uv = self._cmd.process(cdm_bindings.read_pt100, {"ebi": ebi, "bsda": bsda})["ans"]
        uva = uv[0]
        uvb = uv[1]

        r1 = 500
        vcc = 3.37
        vdiff = abs(uva-uvb)/1000.0/1000.0

        rpt100 = (r1 * vdiff + 100 * vcc) / (vcc - vdiff)
        return uv, r1, (rpt100 - 100.0) / 0.39083

    def reg_msg(self, s: str) -> MsgReg:
        return self._data.register(s)



from time import sleep
from pycdm import PyCDM
from pycdm.structs import BsdaSettings, FramingType, EbiSettings

cdm = PyCDM()
cdm.bsda.prepare_ota()
cdm.bsda.flash_ota("/tmp/bsda.bin")
cdm.bsda.boot_ota()
sleep(2)
cdm.bsda.mark_ota_as_ok()
cdm.close()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'form.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1128, 650)
        self.Widget = QtWidgets.QWidget(MainWindow)
        self.Widget.setObjectName("Widget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.Widget)
        self.gridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.verticalLayout.setContentsMargins(10, 10, -1, -1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(self.Widget)
        self.groupBox_2.setStyleSheet("QGroupBox {\n"
"    border: 1px solid gray;\n"
"    border-radius: 9px;\n"
"    margin-top: 0.5em;\n"
"}\n"
"\n"
"QGroupBox::title {\n"
"    subcontrol-origin: margin;\n"
"    left: 10px;\n"
"    padding: 0 3px 0 3px;\n"
"}")
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.groupBox_3 = QtWidgets.QGroupBox(self.groupBox_2)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_6 = QtWidgets.QLabel(self.groupBox_3)
        self.label_6.setObjectName("label_6")
        self.gridLayout_5.addWidget(self.label_6, 0, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.groupBox_3)
        self.label_8.setObjectName("label_8")
        self.gridLayout_5.addWidget(self.label_8, 1, 0, 1, 1)
        self.bsdaL0Label = QtWidgets.QLabel(self.groupBox_3)
        self.bsdaL0Label.setStyleSheet("color: red;")
        self.bsdaL0Label.setObjectName("bsdaL0Label")
        self.gridLayout_5.addWidget(self.bsdaL0Label, 0, 1, 1, 1)
        self.bsdaL1Label = QtWidgets.QLabel(self.groupBox_3)
        self.bsdaL1Label.setStyleSheet("color: red;")
        self.bsdaL1Label.setObjectName("bsdaL1Label")
        self.gridLayout_5.addWidget(self.bsdaL1Label, 1, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.groupBox_3)
        self.label_10.setObjectName("label_10")
        self.gridLayout_5.addWidget(self.label_10, 2, 0, 1, 1)
        self.bsdaL2Label = QtWidgets.QLabel(self.groupBox_3)
        self.bsdaL2Label.setStyleSheet("color: red;")
        self.bsdaL2Label.setObjectName("bsdaL2Label")
        self.gridLayout_5.addWidget(self.bsdaL2Label, 2, 1, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_3, 4, 0, 1, 2)
        self.label_18 = QtWidgets.QLabel(self.groupBox_2)
        self.label_18.setObjectName("label_18")
        self.gridLayout_3.addWidget(self.label_18, 1, 0, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.groupBox_2)
        self.label_20.setObjectName("label_20")
        self.gridLayout_3.addWidget(self.label_20, 2, 0, 1, 1)
        self.label_ebi_left_status = QtWidgets.QLabel(self.groupBox_2)
        self.label_ebi_left_status.setStyleSheet("color: red;")
        self.label_ebi_left_status.setObjectName("label_ebi_left_status")
        self.gridLayout_3.addWidget(self.label_ebi_left_status, 0, 1, 1, 1)
        self.label_ebi_left_version = QtWidgets.QLabel(self.groupBox_2)
        self.label_ebi_left_version.setObjectName("label_ebi_left_version")
        self.gridLayout_3.addWidget(self.label_ebi_left_version, 2, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox_2)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 0, 0, 1, 1)
        self.label_ebi_left_git = QtWidgets.QLabel(self.groupBox_2)
        self.label_ebi_left_git.setObjectName("label_ebi_left_git")
        self.gridLayout_3.addWidget(self.label_ebi_left_git, 1, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.gridLayout_3.addItem(spacerItem, 3, 0, 1, 2)
        self.horizontalLayout.addWidget(self.groupBox_2)
        self.groupBox = QtWidgets.QGroupBox(self.Widget)
        self.groupBox.setStyleSheet("QGroupBox {\n"
"    border: 1px solid gray;\n"
"    border-radius: 9px;\n"
"    margin-top: 0.5em;\n"
"}\n"
"\n"
"QGroupBox::title {\n"
"    subcontrol-origin: margin;\n"
"    left: 10px;\n"
"    padding: 0 3px 0 3px;\n"
"}")
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_22 = QtWidgets.QLabel(self.groupBox)
        self.label_22.setObjectName("label_22")
        self.gridLayout_4.addWidget(self.label_22, 1, 0, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.groupBox)
        self.label_24.setObjectName("label_24")
        self.gridLayout_4.addWidget(self.label_24, 2, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 0, 1, 1)
        self.groupBox_4 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.label_13 = QtWidgets.QLabel(self.groupBox_4)
        self.label_13.setObjectName("label_13")
        self.gridLayout_6.addWidget(self.label_13, 0, 0, 1, 1)
        self.bsdaR0Label = QtWidgets.QLabel(self.groupBox_4)
        self.bsdaR0Label.setStyleSheet("color: red;")
        self.bsdaR0Label.setObjectName("bsdaR0Label")
        self.gridLayout_6.addWidget(self.bsdaR0Label, 0, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.groupBox_4)
        self.label_15.setObjectName("label_15")
        self.gridLayout_6.addWidget(self.label_15, 1, 0, 1, 1)
        self.bsdaR1Label = QtWidgets.QLabel(self.groupBox_4)
        self.bsdaR1Label.setStyleSheet("color: red;")
        self.bsdaR1Label.setObjectName("bsdaR1Label")
        self.gridLayout_6.addWidget(self.bsdaR1Label, 1, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.groupBox_4)
        self.label_17.setObjectName("label_17")
        self.gridLayout_6.addWidget(self.label_17, 2, 0, 1, 1)
        self.bsdaR2Label = QtWidgets.QLabel(self.groupBox_4)
        self.bsdaR2Label.setStyleSheet("color: red;")
        self.bsdaR2Label.setObjectName("bsdaR2Label")
        self.gridLayout_6.addWidget(self.bsdaR2Label, 2, 1, 1, 1)
        self.gridLayout_4.addWidget(self.groupBox_4, 4, 0, 1, 2)
        self.label_ebi_right_version = QtWidgets.QLabel(self.groupBox)
        self.label_ebi_right_version.setObjectName("label_ebi_right_version")
        self.gridLayout_4.addWidget(self.label_ebi_right_version, 2, 1, 1, 1)
        self.label_ebi_right_status = QtWidgets.QLabel(self.groupBox)
        self.label_ebi_right_status.setStyleSheet("color: red;")
        self.label_ebi_right_status.setObjectName("label_ebi_right_status")
        self.gridLayout_4.addWidget(self.label_ebi_right_status, 0, 1, 1, 1)
        self.label_ebi_right_git = QtWidgets.QLabel(self.groupBox)
        self.label_ebi_right_git.setObjectName("label_ebi_right_git")
        self.gridLayout_4.addWidget(self.label_ebi_right_git, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.gridLayout_4.addItem(spacerItem1, 3, 0, 1, 2)
        self.horizontalLayout.addWidget(self.groupBox)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        self.verticalLayout.addItem(spacerItem2)
        self.groupBox_5 = QtWidgets.QGroupBox(self.Widget)
        self.groupBox_5.setStyleSheet("QGroupBox {\n"
"    border: 1px solid gray;\n"
"    border-radius: 9px;\n"
"    margin-top: 0.5em;\n"
"}\n"
"\n"
"QGroupBox::title {\n"
"    subcontrol-origin: margin;\n"
"    left: 10px;\n"
"    padding: 0 3px 0 3px;\n"
"}")
        self.groupBox_5.setFlat(False)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_7.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.groupBox_7 = QtWidgets.QGroupBox(self.groupBox_5)
        self.groupBox_7.setObjectName("groupBox_7")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.groupBox_7)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.label_21 = QtWidgets.QLabel(self.groupBox_7)
        self.label_21.setObjectName("label_21")
        self.gridLayout_9.addWidget(self.label_21, 0, 0, 1, 1)
        self.erFramePeriodMsSpinBox = QtWidgets.QSpinBox(self.groupBox_7)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.erFramePeriodMsSpinBox.sizePolicy().hasHeightForWidth())
        self.erFramePeriodMsSpinBox.setSizePolicy(sizePolicy)
        self.erFramePeriodMsSpinBox.setButtonSymbols(QtWidgets.QAbstractSpinBox.PlusMinus)
        self.erFramePeriodMsSpinBox.setObjectName("erFramePeriodMsSpinBox")
        self.gridLayout_9.addWidget(self.erFramePeriodMsSpinBox, 0, 1, 1, 2)
        self.label_25 = QtWidgets.QLabel(self.groupBox_7)
        self.label_25.setObjectName("label_25")
        self.gridLayout_9.addWidget(self.label_25, 1, 0, 1, 2)
        self.erTemperaturePeriodMsSpinBox = QtWidgets.QSpinBox(self.groupBox_7)
        self.erTemperaturePeriodMsSpinBox.setButtonSymbols(QtWidgets.QAbstractSpinBox.PlusMinus)
        self.erTemperaturePeriodMsSpinBox.setMaximum(65535)
        self.erTemperaturePeriodMsSpinBox.setSingleStep(10)
        self.erTemperaturePeriodMsSpinBox.setObjectName("erTemperaturePeriodMsSpinBox")
        self.gridLayout_9.addWidget(self.erTemperaturePeriodMsSpinBox, 1, 2, 1, 1)
        self.groupBox_9 = QtWidgets.QGroupBox(self.groupBox_7)
        self.groupBox_9.setObjectName("groupBox_9")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_9)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.bsdaR0checkBox = QtWidgets.QCheckBox(self.groupBox_9)
        self.bsdaR0checkBox.setObjectName("bsdaR0checkBox")
        self.verticalLayout_3.addWidget(self.bsdaR0checkBox)
        self.bsdaR1checkBox = QtWidgets.QCheckBox(self.groupBox_9)
        self.bsdaR1checkBox.setObjectName("bsdaR1checkBox")
        self.verticalLayout_3.addWidget(self.bsdaR1checkBox)
        self.bsdaR2checkBox = QtWidgets.QCheckBox(self.groupBox_9)
        self.bsdaR2checkBox.setObjectName("bsdaR2checkBox")
        self.verticalLayout_3.addWidget(self.bsdaR2checkBox)
        self.gridLayout_9.addWidget(self.groupBox_9, 2, 0, 1, 3)
        self.gridLayout_7.addWidget(self.groupBox_7, 8, 2, 1, 1)
        self.frameDurationUsSpinBox = QtWidgets.QSpinBox(self.groupBox_5)
        self.frameDurationUsSpinBox.setButtonSymbols(QtWidgets.QAbstractSpinBox.PlusMinus)
        self.frameDurationUsSpinBox.setMaximum(1000)
        self.frameDurationUsSpinBox.setSingleStep(10)
        self.frameDurationUsSpinBox.setObjectName("frameDurationUsSpinBox")
        self.gridLayout_7.addWidget(self.frameDurationUsSpinBox, 5, 2, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.groupBox_5)
        self.label_5.setObjectName("label_5")
        self.gridLayout_7.addWidget(self.label_5, 6, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.groupBox_5)
        self.label_4.setObjectName("label_4")
        self.gridLayout_7.addWidget(self.label_4, 5, 1, 1, 1)
        self.groupBox_6 = QtWidgets.QGroupBox(self.groupBox_5)
        self.groupBox_6.setObjectName("groupBox_6")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBox_6)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.label_19 = QtWidgets.QLabel(self.groupBox_6)
        self.label_19.setObjectName("label_19")
        self.gridLayout_8.addWidget(self.label_19, 0, 0, 1, 1)
        self.elTemperaturePeriodMsSpinBox = QtWidgets.QSpinBox(self.groupBox_6)
        self.elTemperaturePeriodMsSpinBox.setButtonSymbols(QtWidgets.QAbstractSpinBox.PlusMinus)
        self.elTemperaturePeriodMsSpinBox.setMaximum(65535)
        self.elTemperaturePeriodMsSpinBox.setSingleStep(10)
        self.elTemperaturePeriodMsSpinBox.setObjectName("elTemperaturePeriodMsSpinBox")
        self.gridLayout_8.addWidget(self.elTemperaturePeriodMsSpinBox, 1, 1, 1, 1)
        self.elFramePeriodMsSpinBox = QtWidgets.QSpinBox(self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.elFramePeriodMsSpinBox.sizePolicy().hasHeightForWidth())
        self.elFramePeriodMsSpinBox.setSizePolicy(sizePolicy)
        self.elFramePeriodMsSpinBox.setButtonSymbols(QtWidgets.QAbstractSpinBox.PlusMinus)
        self.elFramePeriodMsSpinBox.setObjectName("elFramePeriodMsSpinBox")
        self.gridLayout_8.addWidget(self.elFramePeriodMsSpinBox, 0, 1, 1, 1)
        self.label_23 = QtWidgets.QLabel(self.groupBox_6)
        self.label_23.setObjectName("label_23")
        self.gridLayout_8.addWidget(self.label_23, 1, 0, 1, 1)
        self.groupBox_8 = QtWidgets.QGroupBox(self.groupBox_6)
        self.groupBox_8.setObjectName("groupBox_8")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_8)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.bsdaL0checkBox = QtWidgets.QCheckBox(self.groupBox_8)
        self.bsdaL0checkBox.setObjectName("bsdaL0checkBox")
        self.verticalLayout_2.addWidget(self.bsdaL0checkBox)
        self.bsdaL1checkBox = QtWidgets.QCheckBox(self.groupBox_8)
        self.bsdaL1checkBox.setObjectName("bsdaL1checkBox")
        self.verticalLayout_2.addWidget(self.bsdaL1checkBox)
        self.bsdaL2checkBox = QtWidgets.QCheckBox(self.groupBox_8)
        self.bsdaL2checkBox.setObjectName("bsdaL2checkBox")
        self.verticalLayout_2.addWidget(self.bsdaL2checkBox)
        self.gridLayout_8.addWidget(self.groupBox_8, 2, 0, 1, 1)
        self.gridLayout_7.addWidget(self.groupBox_6, 8, 1, 1, 1)
        self.enableTemperatureCheckBox = QtWidgets.QCheckBox(self.groupBox_5)
        self.enableTemperatureCheckBox.setObjectName("enableTemperatureCheckBox")
        self.gridLayout_7.addWidget(self.enableTemperatureCheckBox, 0, 1, 1, 1)
        self.temperaturePeriodMsSpinBox = QtWidgets.QSpinBox(self.groupBox_5)
        self.temperaturePeriodMsSpinBox.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.temperaturePeriodMsSpinBox.setButtonSymbols(QtWidgets.QAbstractSpinBox.PlusMinus)
        self.temperaturePeriodMsSpinBox.setMaximum(90000)
        self.temperaturePeriodMsSpinBox.setSingleStep(100)
        self.temperaturePeriodMsSpinBox.setObjectName("temperaturePeriodMsSpinBox")
        self.gridLayout_7.addWidget(self.temperaturePeriodMsSpinBox, 6, 2, 1, 1)
        self.enableLightCheckBox = QtWidgets.QCheckBox(self.groupBox_5)
        self.enableLightCheckBox.setObjectName("enableLightCheckBox")
        self.gridLayout_7.addWidget(self.enableLightCheckBox, 3, 1, 1, 1)
        self.enableImuCheckBox = QtWidgets.QCheckBox(self.groupBox_5)
        self.enableImuCheckBox.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.enableImuCheckBox.setAutoFillBackground(False)
        self.enableImuCheckBox.setObjectName("enableImuCheckBox")
        self.gridLayout_7.addWidget(self.enableImuCheckBox, 2, 1, 1, 1)
        self.framingTypeComboBox = QtWidgets.QComboBox(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.framingTypeComboBox.sizePolicy().hasHeightForWidth())
        self.framingTypeComboBox.setSizePolicy(sizePolicy)
        self.framingTypeComboBox.setObjectName("framingTypeComboBox")
        self.framingTypeComboBox.addItem("")
        self.framingTypeComboBox.addItem("")
        self.framingTypeComboBox.addItem("")
        self.framingTypeComboBox.addItem("")
        self.framingTypeComboBox.addItem("")
        self.gridLayout_7.addWidget(self.framingTypeComboBox, 4, 2, 1, 1)
        self.label_26 = QtWidgets.QLabel(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_26.sizePolicy().hasHeightForWidth())
        self.label_26.setSizePolicy(sizePolicy)
        self.label_26.setObjectName("label_26")
        self.gridLayout_7.addWidget(self.label_26, 4, 1, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_5)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 2, 2, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem3, 0, 1, 2, 1)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.gridLayout.setObjectName("gridLayout")
        self.connectCdmButton = QtWidgets.QPushButton(self.Widget)
        self.connectCdmButton.setMinimumSize(QtCore.QSize(0, 33))
        self.connectCdmButton.setObjectName("connectCdmButton")
        self.gridLayout.addWidget(self.connectCdmButton, 2, 0, 1, 2)
        self.serverEdit = QtWidgets.QLineEdit(self.Widget)
        self.serverEdit.setObjectName("serverEdit")
        self.gridLayout.addWidget(self.serverEdit, 0, 1, 1, 1)
        self.startButton = QtWidgets.QPushButton(self.Widget)
        self.startButton.setEnabled(False)
        self.startButton.setMinimumSize(QtCore.QSize(0, 33))
        self.startButton.setObjectName("startButton")
        self.gridLayout.addWidget(self.startButton, 8, 0, 1, 2)
        self.stopButton = QtWidgets.QPushButton(self.Widget)
        self.stopButton.setEnabled(False)
        self.stopButton.setMinimumSize(QtCore.QSize(0, 33))
        self.stopButton.setObjectName("stopButton")
        self.gridLayout.addWidget(self.stopButton, 9, 0, 1, 2)
        spacerItem4 = QtWidgets.QSpacerItem(20, 423, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Ignored)
        self.gridLayout.addItem(spacerItem4, 3, 0, 1, 2)
        self.svgContainer = QtWidgets.QWidget(self.Widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.svgContainer.sizePolicy().hasHeightForWidth())
        self.svgContainer.setSizePolicy(sizePolicy)
        self.svgContainer.setMinimumSize(QtCore.QSize(250, 250))
        self.svgContainer.setBaseSize(QtCore.QSize(100, 100))
        self.svgContainer.setObjectName("svgContainer")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.svgContainer)
        self.gridLayout_10.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.gridLayout.addWidget(self.svgContainer, 5, 0, 1, 2)
        self.configureButton = QtWidgets.QPushButton(self.Widget)
        self.configureButton.setEnabled(False)
        self.configureButton.setMinimumSize(QtCore.QSize(0, 33))
        self.configureButton.setObjectName("configureButton")
        self.gridLayout.addWidget(self.configureButton, 7, 0, 1, 2)
        self.label = QtWidgets.QLabel(self.Widget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.connectEbiButton = QtWidgets.QPushButton(self.Widget)
        self.connectEbiButton.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.connectEbiButton.sizePolicy().hasHeightForWidth())
        self.connectEbiButton.setSizePolicy(sizePolicy)
        self.connectEbiButton.setMinimumSize(QtCore.QSize(0, 33))
        self.connectEbiButton.setObjectName("connectEbiButton")
        self.gridLayout.addWidget(self.connectEbiButton, 6, 0, 1, 1)
        self.disconnectEbiButton = QtWidgets.QPushButton(self.Widget)
        self.disconnectEbiButton.setEnabled(False)
        self.disconnectEbiButton.setMinimumSize(QtCore.QSize(0, 33))
        self.disconnectEbiButton.setObjectName("disconnectEbiButton")
        self.gridLayout.addWidget(self.disconnectEbiButton, 6, 1, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 2, 1)
        MainWindow.setCentralWidget(self.Widget)
        self.statusBar = QtWidgets.QStatusBar(MainWindow)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        self.Widget.setWindowTitle(_translate("MainWindow", "Widget"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Ebi Left"))
        self.groupBox_3.setTitle(_translate("MainWindow", "BSDAs"))
        self.label_6.setText(_translate("MainWindow", "L0:"))
        self.label_8.setText(_translate("MainWindow", "L1:"))
        self.bsdaL0Label.setText(_translate("MainWindow", "DISCONNECTED"))
        self.bsdaL1Label.setText(_translate("MainWindow", "DISCONNECTED"))
        self.label_10.setText(_translate("MainWindow", "L2:"))
        self.bsdaL2Label.setText(_translate("MainWindow", "DISCONNECTED"))
        self.label_18.setText(_translate("MainWindow", "Git:"))
        self.label_20.setText(_translate("MainWindow", "Version:"))
        self.label_ebi_left_status.setText(_translate("MainWindow", "DISCONNECTED"))
        self.label_ebi_left_version.setText(_translate("MainWindow", "NOE"))
        self.label_2.setText(_translate("MainWindow", "Status:"))
        self.label_ebi_left_git.setText(_translate("MainWindow", "NONE"))
        self.groupBox.setTitle(_translate("MainWindow", "Ebi Right"))
        self.label_22.setText(_translate("MainWindow", "Git:"))
        self.label_24.setText(_translate("MainWindow", "Version:"))
        self.label_3.setText(_translate("MainWindow", "Status:"))
        self.groupBox_4.setTitle(_translate("MainWindow", "BSDAs"))
        self.label_13.setText(_translate("MainWindow", "R0:"))
        self.bsdaR0Label.setText(_translate("MainWindow", "DISCONNECTED"))
        self.label_15.setText(_translate("MainWindow", "R1:"))
        self.bsdaR1Label.setText(_translate("MainWindow", "DISCONNECTED"))
        self.label_17.setText(_translate("MainWindow", "R2:"))
        self.bsdaR2Label.setText(_translate("MainWindow", "DISCONNECTED"))
        self.label_ebi_right_version.setText(_translate("MainWindow", "NONE"))
        self.label_ebi_right_status.setText(_translate("MainWindow", "DISCONNECTED"))
        self.label_ebi_right_git.setText(_translate("MainWindow", "NONE"))
        self.groupBox_5.setTitle(_translate("MainWindow", "Configuration"))
        self.groupBox_7.setTitle(_translate("MainWindow", "Ebi right"))
        self.label_21.setText(_translate("MainWindow", "Frame reader period (in milliseconds): "))
        self.label_25.setText(_translate("MainWindow", "Temperature reader period (in milliseconds):"))
        self.groupBox_9.setTitle(_translate("MainWindow", "BSDAs enabled"))
        self.bsdaR0checkBox.setText(_translate("MainWindow", "R0"))
        self.bsdaR1checkBox.setText(_translate("MainWindow", "R1"))
        self.bsdaR2checkBox.setText(_translate("MainWindow", "R2"))
        self.label_5.setText(_translate("MainWindow", "BSDA temperature period (in milliseconds):"))
        self.label_4.setText(_translate("MainWindow", "BSDA frame duration (in microseconds):"))
        self.groupBox_6.setTitle(_translate("MainWindow", "Ebi left"))
        self.label_19.setText(_translate("MainWindow", "Frame reader period (in milliseconds):"))
        self.label_23.setText(_translate("MainWindow", "Temperature reader period (in milliseconds):"))
        self.groupBox_8.setTitle(_translate("MainWindow", "BSDAs enabled"))
        self.bsdaL0checkBox.setText(_translate("MainWindow", "L0"))
        self.bsdaL1checkBox.setText(_translate("MainWindow", "L1"))
        self.bsdaL2checkBox.setText(_translate("MainWindow", "L2"))
        self.enableTemperatureCheckBox.setText(_translate("MainWindow", "Enable temperature reading"))
        self.enableLightCheckBox.setText(_translate("MainWindow", "Enable light reading"))
        self.enableImuCheckBox.setText(_translate("MainWindow", "Enable IMU"))
        self.framingTypeComboBox.setItemText(0, _translate("MainWindow", "Full "))
        self.framingTypeComboBox.setItemText(1, _translate("MainWindow", "Sum"))
        self.framingTypeComboBox.setItemText(2, _translate("MainWindow", "Sum squared"))
        self.framingTypeComboBox.setItemText(3, _translate("MainWindow", "Histogram"))
        self.framingTypeComboBox.setItemText(4, _translate("MainWindow", "Test"))
        self.label_26.setText(_translate("MainWindow", "Framing type:"))
        self.connectCdmButton.setText(_translate("MainWindow", "Connect to CDM"))
        self.startButton.setText(_translate("MainWindow", "Start"))
        self.stopButton.setText(_translate("MainWindow", "Stop"))
        self.configureButton.setText(_translate("MainWindow", "Configure"))
        self.label.setText(_translate("MainWindow", "CDM Server"))
        self.connectEbiButton.setText(_translate("MainWindow", "Connect to EBIs"))
        self.disconnectEbiButton.setText(_translate("MainWindow", "Disconnect from EBI"))



import sys

from PyQt5 import QtSvg, Qt, QtCore
from PyQt5.QtWidgets import QApplication, QMainWindow
from statemachine import StateMachine, State
from svgelements import SVG
from pyqt_tools import messages

import cdm_bindings
from pycdm import PyCDM, Cfg
from pycdm.structs import BsdaSettings, FramingType, EbiSettings
from .ui_form import Ui_MainWindow


class PyCdmStateMachine(StateMachine):
    disconnected = State(initial=True)
    cdm_connected = State()
    ebi_connected = State()
    bcg_configured = State()

    connect_cdm = (disconnected.to(cdm_connected))
    connect_ebis = (cdm_connected.to(ebi_connected))
    configure_bcg = (cdm_connected.to(bcg_configured))
    disconnect_ebis = (ebi_connected.to(cdm_connected))

    def __init__(self, widget):
        super().__init__()
        self._w = widget
        self._previous_properties = {}
        # After restoring pixmap property the text disappears. As a workaround create a list of properties
        # that we change and restore only those.
        self._valid_properties = ["enabled", "text", "styleSheet"]

    def on_enter_cdm_connected(self, v = None):
        self._w.ui.connectEbiButton.setEnabled(True)
        self._w.ui.connectCdmButton.setDisabled(True)
        if v:
            self._w.statusBar().showMessage("Running CDM version {} with git {} ".format(v["version"], v["git"]))

    def on_enter_ebi_connected(self, e, s):
        self._store_properties()
        self._w.ui.connectEbiButton.setDisabled(True)
        self._w.ui.disconnectEbiButton.setEnabled(True)
        self._w.ui.configureButton.setEnabled(True)

        el = e["ebi_left"]
        self._w.ui.label_ebi_left_git.setText(el["git"])
        self._w.ui.label_ebi_left_version.setText(el["version"])
        self._w.ui.label_ebi_left_status.setStyleSheet("color: green;")
        self._w.ui.label_ebi_left_status.setText("CONNECTED")

        er = e["ebi_right"]
        self._w.ui.label_ebi_right_git.setText(er["git"])
        self._w.ui.label_ebi_right_version.setText(er["version"])

        bcg_left = s["left"]
        bcg_right = s["right"]

        def getEbiTextAndColor(ebi_status):
            ebi_color = "color: green;"
            if ebi_status == 0:
                ebi_text = "Initialized"
            elif ebi_status == 1:
                ebi_text = "Ready"
            elif ebi_status == 2:
                ebi_text = "Reading"
            elif ebi_status == 3:
                ebi_text = "Dead"
                ebi_color = "color: read;"
            else:
                ebi_text = "Unkown"
                ebi_color = "color: read;"

            return ebi_text, ebi_color

        text, color = getEbiTextAndColor(bcg_left["ebi"])
        self._w.ui.label_ebi_right_status.setText(text.upper())
        self._w.ui.label_ebi_right_status.setStyleSheet(color)

        text, color = getEbiTextAndColor(bcg_right["ebi"])
        self._w.ui.label_ebi_right_status.setText(text.upper())
        self._w.ui.label_ebi_right_status.setStyleSheet(color)

        def get_bsda_text_and_color(bsda_status):
            bsda_color = "color: green;"
            if bsda_status == 0:
                bsda_text = "Initialized"
            elif bsda_status == 1:
                bsda_text = "Ready"
            elif bsda_status == 2:
                bsda_text = "Reading"
            elif bsda_status == 3:
                bsda_text = "Dead"
                bsda_color = "color: red;"
            elif bsda_status == 4:
                bsda_text = "Disabled"
                bsda_color = "color: yellow;"
            else:
                bsda_text = "Unkown"
                bsda_color = "color: read;"

            return bsda_text, bsda_color

        for i in range(3):
            text, color = get_bsda_text_and_color(bcg_left["bsda"][i])
            tmp = getattr(self._w.ui, "bsdaL{}Label".format(i))
            tmp.setText(text.upper())
            tmp.setStyleSheet(color)

        for i in range(3):
            text, color = get_bsda_text_and_color(bcg_right["bsda"][i])
            tmp = getattr(self._w.ui, "bsdaR{}Label".format(i))
            tmp.setText(text.upper())
            tmp.setStyleSheet(color)

    def on_exit_ebi_connected(self):
        self._restore_properties()

    def on_enter_bcg_configured(self):
        self._w.ui.disconnectEbiButton.setEnabled(True)
        self._w.ui.startButton.setEnabled(True)

    def _store_properties(self):
        for i in self._w.ui.__dict__:
            v = self._w.ui.__dict__[i]
            mo = v.metaObject()
            for j in range(mo.propertyCount()):
                pn = mo.property(j).name()
                if pn in self._valid_properties:
                    key = "{}_{}".format(v.objectName(), pn)
                    self._previous_properties[key] = mo.property(j).read(v)

    def _restore_properties(self):
        for i in self._w.ui.__dict__:
            v = self._w.ui.__dict__[i]
            mo = v.metaObject()
            for j in range(mo.propertyCount()):
                pn = mo.property(j).name()
                if pn in self._valid_properties:
                    key = "{}_{}".format(v.objectName(), pn)
                    v.setProperty(pn, self._previous_properties[key])


class Widget(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._cdm = None
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.ui.connectCdmButton.clicked.connect(self._connect_cdm)
        self.ui.connectEbiButton.clicked.connect(self._connect_ebis)
        self.ui.disconnectEbiButton.clicked.connect(self._disconnect_ebis)
        self.ui.configureButton.clicked.connect(self._configure_bcg)
        self.ui.serverEdit.setText(Cfg().get_cdm_addr())
        self._sm = PyCdmStateMachine(self)

        self._svg_widget = QtSvg.QSvgWidget("/tmp/cdm.svg", self.ui.svgContainer)
        self.ui.svgContainer.adjustSize()
        self._svg_widget.resize(Qt.QSize(self.ui.svgContainer.minimumSize()))
        self.ui.svgContainer.update()
        self._svg_widget.update()
        self._svg_widget.show()

    def closeEvent(self, event):
        if self._cdm:
            self._cdm.close()

    def _connect_cdm(self):
        try:
            def send_connect():
                e = self._cdm.ebi.version()
                s = self._cdm.bcg.get_state()
                self._sm.send("connect_ebis", e, s)

            self._cdm = PyCDM(self.ui.serverEdit.text())
            v = self._cdm.cdm.get_info()
            self._sm.send("connect_cdm", v)

            state = self._cdm.cdm.get_sm_state()
            if state == cdm_bindings.SM_STATE.ERBI_READY:
                send_connect()
            elif state == cdm_bindings.SM_STATE.BSDA_READY:
                send_connect()
                self._sm.send("configure_bcg")

        except Exception as e:
            messages.show_fatal(e, "Fatal error")

    def _connect_ebis(self):
        try:
            e = self._cdm.ebi.connect()
            # e = {'ebi_left': {'git': 'heads/dev-0-g027d9c1-dirty', 'name': 'erbi', 'version': '0.0.1'}, 'ebi_right': {'git': 'heads/dev-0-g027d9c1-dirty', 'name': 'erbi', 'version': '0.0.1'}}
            s = self._cdm.bcg.get_state()
            self._sm.send("connect_ebis", e, s)
        except Exception as e:
            messages.show_fatal(e, "Fatal error")

    def _disconnect_ebis(self):
        try:
            self._cdm.ebi.disconnect()
        except Exception as e:
            messages.show_fatal(e, "Fatal error")
        self._sm.send("disconnect_ebis")

    def _configure_bcg(self):
        try:
            bsda = BsdaSettings()
            selected_framing_type = self.ui.framingTypeComboBox.currentText().upper()
            if selected_framing_type == "FULL":
                bsda.framing = FramingType.FULL
            elif selected_framing_type == "SUM":
                bsda.framing = FramingType.SUM
            elif selected_framing_type == "HISTOGRAM":
                bsda.framing = FramingType.HISTOGRAM
            elif selected_framing_type == "TEST":
                bsda.framing = FramingType.TEST
            elif selected_framing_type == "SUM SQUARED":
                bsda.framing = FramingType.SUM_SQ
            else:
                raise Exception("Invalid framing type found \"{}\"".format(selected_framing_type))

            bsda.frame_duration_us = self.ui.frameDurationUsSpinBox.value()
            bsda.temperature_period_ms = self.ui.temperaturePeriodMsSpinBox.value()
            bsda.disabled = False
            bsda.light_enabled = self.ui.enableLightCheckBox.isEnabled()
            bsda.temperature_enabled = self.ui.enableTemperatureCheckBox.isEnabled()
            # TODO: Allow to configure ADC shadow
            bsda.adc_shadow = [0xf9f0, 0xf9f0]
            bsda.log_uart = True
            bsda.log_level = 3

            bsda_disabled = BsdaSettings()
            bsda_disabled.disabled = True

            left = EbiSettings()
            left.frame_reader_framing_type = bsda.framing
            left.frame_reader_period_ms = self.ui.elFramePeriodMsSpinBox.value()
            left.temperature_reader_period_ms = self.ui.elTemperaturePeriodMsSpinBox.value()
            left.bsda_settings[0] = bsda if self.ui.bsdaL0checkBox.isEnabled() else bsda_disabled
            left.bsda_settings[1] = bsda if self.ui.bsdaL1checkBox.isEnabled() else bsda_disabled
            left.bsda_settings[2] = bsda if self.ui.bsdaL2checkBox.isEnabled() else bsda_disabled
            left.log_uart = bsda.log_uart
            left.log_level = bsda.log_level

            right = EbiSettings()
            right.frame_reader_framing_type = bsda.framing
            right.frame_reader_period_ms = self.ui.erFramePeriodMsSpinBox.value()
            right.temperature_reader_period_ms = self.ui.erTemperaturePeriodMsSpinBox.value()
            right.bsda_settings[0] = bsda if self.ui.bsdaR0checkBox.isEnabled() else bsda_disabled
            right.bsda_settings[1] = bsda if self.ui.bsdaR1checkBox.isEnabled() else bsda_disabled
            right.bsda_settings[2] = bsda if self.ui.bsdaR2CheckBox.isEnabled() else bsda_disabled
            right.log_uart = bsda.log_uart
            right.log_level = bsda.log_level
            self._cdm.configure(left, right)
        except Exception as e:
            messages.show_fatal(e, "Fatal error")

        self._sm.send("configure_bcg")


def main():
    app = QApplication(sys.argv)
    widget = Widget()
    widget.show()
    sys.exit(app.exec())


if __name__ == "__main__":
    main()



import socket

import zmq

context: zmq.Context = zmq.Context()


class ZmqSyncConn:
    def __init__(self, cdm_addr: str) -> None:
        t = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        t.settimeout(2)
        if t.connect_ex((cdm_addr, 32000)) != 0:
            raise Exception("CDM doesn't seems to be running")
        self._sock = context.socket(zmq.REQ)
        self._sock.connect("tcp://{}:32000".format(cdm_addr))

    def send(self, s: str) -> None:
        self._sock.send_string(s)

    def recv(self) -> bytes:
        return self._sock.recv()


class ZmqAsyncConn:
    def __init__(self, cdm_addr: str) -> None:
        self._sock = context.socket(zmq.SUB)
        self._ctxt = self._sock.connect("tcp://{}:32001".format(cdm_addr))

    def recv_msg(self) -> tuple[str, bytes]:
        msg_id = self._sock.recv().decode()
        msg = self._sock.recv()
        return msg_id, msg

    def subscribe(self, topic: str) -> None:
        self._sock.setsockopt_string(zmq.SUBSCRIBE, topic)

    def disconnect(self) -> None:
        global context
        context.destroy()
        context = zmq.Context()



import tomli
from platformdirs import user_config_dir
from typing import Any

class Cfg:
    def __init__(self) -> None:
        appname = "cdm"
        cfg_dir = user_config_dir(appname)
        cfg_file = cfg_dir + "/frontend.toml"

        with open(cfg_file, mode="rb") as fp:
            self._cfg: dict[str, Any] = tomli.load(fp)

    def get_cdm_addr(self) -> str:
        return self._cfg["cdm"]



from enum import Enum, auto
from typing import Any


class FramingType(Enum):
    FULL = 0
    SUM = auto()
    RAW = auto()
    HISTOGRAM = auto()
    TEST = auto()
    SUM_SQ = auto()


class BsdaSettings:

    def __init__(self) -> None:
        self.framing: FramingType = FramingType.SUM
        # s.bsda_settings[1].frame_duration_us = (f != FRAMING_T::HISTOGRAM ? 1000: 10000);
        self.frame_duration_us: int = 0
        self.temperature_period_ms: int = 0
        self.voltage_period_ms: int = 5000
        # default frequency is SPI_MASTER_FREQ_16M
        # Possible values are
        # SPI_MASTER_FREQ_8M      (80 * 1000 * 1000 / 10)   ///< 8MHz
        # SPI_MASTER_FREQ_9M      (80 * 1000 * 1000 / 9)    ///< 8.89MHz
        # SPI_MASTER_FREQ_10M     (80 * 1000 * 1000 / 8)    ///< 10MHz
        # SPI_MASTER_FREQ_11M     (80 * 1000 * 1000 / 7)    ///< 11.43MHz
        # SPI_MASTER_FREQ_13M     (80 * 1000 * 1000 / 6)    ///< 13.33MHz
        # SPI_MASTER_FREQ_16M     (80 * 1000 * 1000 / 5)    ///< 16MHz
        # SPI_MASTER_FREQ_20M     (80 * 1000 * 1000 / 4)    ///< 20MHz
        # SPI_MASTER_FREQ_26M     (80 * 1000 * 1000 / 3)    ///< 26.67MHz
        # SPI_MASTER_FREQ_40M     (80 * 1000 * 1000 / 2)    ///< 40MHz
        # SPI_MASTER_FREQ_80M     (80 * 1000 * 1000 / 1)    ///< 80MHz
        self.adc_freq_khz: list[int] = [16000, 16000]
        self.adc_shadow: list[int] = [0x03ff, 0x03ff]
        self.raw_adc: int = 0
        self.raw_channel: int = 0
        self.log_uart: bool = False
        self.log_level: int = 0
        self.disabled: bool = False
        self.light_enabled: bool = False
        self.temperature_enabled: bool = False
        self.voltage_enabled: bool = False

    def to_json(self) -> dict[str, Any]:
        if self.temperature_period_ms > (2**16)-1:
            raise ValueError("Temperature period is greater than 2**16 - 1")

        d: dict[str, Any] = {}
        d["framing"] = self.framing.value
        d["frame_duration_us"] = self.frame_duration_us
        d["temperature_period"] = self.temperature_period_ms
        d["voltage_period"] = self.voltage_period_ms
        d["adc_freq"] = self.adc_freq_khz
        d["adc_shadow"] = self.adc_shadow
        d["raw_adc"] = self.raw_adc
        d["raw_channel"] = self.raw_channel
        d["log_uart"] = self.log_uart
        d["log_level"] = self.log_level
        d["disabled"] = self.disabled
        d["light_enabled"] = self.light_enabled
        d["temperature_enabled"] = self.temperature_enabled
        d["voltage_enabled"] = self.voltage_enabled
        return d

    def from_json(self, j: dict[str, Any]) -> "BsdaSettings":
        self.framing = FramingType(j["framing"])
        self.frame_duration_us = j["frame_duration_us"]
        self.temperature_period_ms = j["temperature_period"]
        self.voltage_period_ms = j["voltage_period"]
        self.adc_freq_khz = j["adc_freq"]
        self.adc_shadow = j["adc_shadow"]
        self.raw_adc = j["raw_adc"]
        self.raw_channel = j["raw_channel"]
        self.log_uart = j["log_uart"]
        self.log_level = j["log_level"]
        self.disabled = j["disabled"]
        self.light_enabled = j["light_enabled"]
        self.temperature_enabled = j["temperature_enabled"]
        self.voltage_enabled = j["voltage_enabled"]

        return self

class EbiSettings:

    def __init__(self) -> None:
        #s.framereader_period = (f != FRAMING_T::HISTOGRAM ? 10: 100);
        self.voltage_reader_period_ms: int = 5000
        self.temperature_reader_period_ms: int = 0
        self.frame_reader_period_ms: int = 0
        self.frame_reader_framing_type: FramingType = FramingType.SUM
        self.raw_board: int = 0
        self.bsda_settings: list[BsdaSettings] = [BsdaSettings(), BsdaSettings(), BsdaSettings()]
        self.ntp_period_m: int = 0
        self.log_uart: bool = False
        self.log_level: int = 0

    def to_json(self) -> dict[str, Any]:
        d: dict[str, Any] = {}
        d["voltage_reader_period"] = self.voltage_reader_period_ms
        d["temperature_reader_period"] = self.temperature_reader_period_ms
        d["period"] = self.frame_reader_period_ms
        d["type"] = self.frame_reader_framing_type.value
        d["bsda_settings"] = [s.to_json() for s in self.bsda_settings]
        d["raw_board"] = self.raw_board
        d["ntp_period_m"] = self.ntp_period_m
        d["log_level"] = self.log_level
        d["log_uart"] = self.log_uart
        return d

    def from_json(self, j: dict[str, Any]) -> "EbiSettings":
        self.voltage_reader_period_ms = j["voltage_reader_period"]
        self.temperature_reader_period_ms = j["temperature_reader_period"]
        self.frame_reader_period_ms = j["period"]
        self.frame_reader_framing_type = FramingType(j["type"])
        self.bsda_settings = [BsdaSettings().from_json(e) for e in j["bsda_settings"]]
        self.raw_board = j["raw_board"]
        self.ntp_period_m = j["ntp_period_m"]
        self.log_level = j["log_level"]
        self.log_uart = j["log_uart"]

        return self



from pycdm import PyCDM
from pycdm.structs import BsdaSettings, FramingType, EbiSettings
from time import sleep
cdm = PyCDM("127.0.0.1")
cdm.ebi.connect()
cdm.ebi.flash_ota("/tmp/erbi.bin")
cdm.ebi.boot_ota()
sleep(2)
cdm.ebi.mark_ota_as_ok()
cdm.close()



import sys
from struct import unpack, calcsize

import cdm_bindings

if len(sys.argv) != 2:
    print("Expected file path as an argument")
    sys.exit(-1)

with open(sys.argv[1], "rb") as f:
    s = f.read()

decoder = cdm_bindings.FileDecoder(s)
ebis = decoder.get_baffle_settings()
header = decoder.get_header()
print(hex(header.magic_byte))
print(header.type)
frame = decoder.read_full_frame();
print(frame.timestamp)
print(frame.count, len(frame.data))
print(frame.data[0].sum)




from setuptools import setup, find_packages, Extension

setup(
    name = 'virgo-pycdm',
    packages = find_packages(),
    platforms=['any'],
    entry_points = {
        'console_scripts': ['cdm_gui=cdm_gui.widget:main'],
    },
    version = '0.5.0',
    author = 'David Roman',
    author_email = 'droman@ifae.es',
    url = 'https://gitlab.pic.es/virgo-sw/cdm-python-frontend',
    license = 'BSD-3',
    include_package_data=True,
    package_data={"pycdm": ["py.typed"]},
    install_requires=[
            'platformdirs',
            'PyQt5',
            'pybind11',
            'pyqt_tools',
            'pyzmq',
            'tomli',
        ],
	classifiers=[
		'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
		'Programming Language :: Python :: 3'
	],
    ext_modules=[
        Extension(
            name="cdm_bindings",  # as it would be imported
            sources=["common-cdm/cdm_bindings.cpp", "common-cdm/frame_decoding.cpp"], # all sources are compiled into a single binary file
            extra_link_args=["-lfmt"],
            extra_compile_args=["-std=c++20"],
        ),
    ]
)



import json
from typing import Any

import cdm_bindings

from .comm import ZmqSyncConn
from .structs import EbiSettings
from .utils import *

"""
for the list of implemented commands see cmd.cpp:CmdExecutor::execute
"""
class CmdUtils:
    def __init__(self, addr: str) -> None:
        self._cdm_cmd: ZmqSyncConn = ZmqSyncConn(addr)

    def flash(self, fname: str, cmd: str) -> dict[str, Any]:
        fw, h = encode_from_file(fname)
        args = {"hash": h, "firmware_data": fw.decode()}
        return self.process(cmd, args)

    def process(self, cmd: str, args: dict[str, Any] = {}) -> dict[str, Any]:
        msg = json.dumps({"cmd": cmd, "args": args})
        self._cdm_cmd.send(msg)

        ans = json.loads(self._cdm_cmd.recv())
        if ans is None:
            return {}

        err = ans.pop("error", None)
        if err is not None:
            raise Exception(err)

        return ans

    def download(self, cmd: str, dst: str, key: str, args: dict[str, Any] = {}) -> dict[str, Any]:
        ans = self.process(cmd, args)
        decode_to_file(ans[key], dst)
        del ans[key]
        return ans


class CdmCmd:
    def __init__(self, cmd: CmdUtils) -> None:
        self._cmd: CmdUtils = cmd

    def get_info(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.cdm_version)

    def get_sm_state(self) -> Any:
        return cdm_bindings.SM_STATE(self._cmd.process(cdm_bindings.get_cdm_state)["state"])

    def get_config(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_cdm_config)

    def download_file(self, year: int, month: int, day: int, hour: int, fname: str, dpath: str) -> dict[str, Any]:
        d = {"year": year, "month": month, "day": day, "hour": hour, "file_name": fname}
        return self._cmd.download(cdm_bindings.download_file, dpath, "file_content", d)

    def power_on(self, s: str = "") -> dict[str, Any]:
        if s != "I_KNOW_WHAT_I_AM_DOING":
            raise Exception("Not allowed to use this method")

        return self._cmd.process(cdm_bindings.power_on, {"is_engineer": True})

    def power_off(self, s: str = "") -> dict[str, Any]:
        if s != "I_KNOW_WHAT_I_AM_DOING":
            raise Exception("Not allowed to use this method")

        return self._cmd.process(cdm_bindings.power_off, {"is_engineer": True})

    def get_warnings(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_warnings)


"""
Bcg stands for "Baffle control group". A BCG is the combination of 1 ERBI plus 3 BSDAs.
There are two BCG each one controlled by one ERBI (or a concentrator BSDA).
"""

class BcgCmd:
    def __init__(self, cmd: CmdUtils) -> None:
        self._cmd: CmdUtils = cmd

    def get_running_partition(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_partition)

    def get_info(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_info)

    def get_voltage(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_voltage)

    def get_adc_map(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_adc_map)

    def get_adc_freq(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_adc_freq)

    def adc_oneshot(self, b: str, adc: int, ch: int) -> Any:
        args = {"board_identifier": b, "adc": adc, "ch": ch}
        return self._cmd.process(cdm_bindings.adc_oneshot, args)["val"]

    def get_config(self) -> tuple[EbiSettings, EbiSettings]:
        s = self._cmd.process(cdm_bindings.get_config)
        left = EbiSettings().from_json(s["left"])
        right = EbiSettings().from_json(s["right"])
        return left, right

    def get_state(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_state)

    def dump_log(self, idns: list[str]) -> dict[str, list[str]]:
        """
        :param idns: List of valid identifiers. Can be any of {R,L} or {R,L}{0,1,2}
        """
        for i in idns:
            check_valid_board_identifier(i)

        args = {"board_identifiers": idns}
        ans = self._cmd.process(cdm_bindings.dump_log, args)

        for k, v in ans.items():
            ans[k] = v.splitlines()

        return ans

    def hard_reset(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.hard_reset, {"reset_erbi": True, "reset_bsdas": True})

    def versions(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.get_version)


class EbisCmd:
    def __init__(self, cmd: CmdUtils) -> None:
        self._cmd: CmdUtils = cmd

    # XXX: Maybe this function should be moved to CDM class as it's the CDM who is connecting to the EBI
    # the EBI is not connecting anywhere
    def connect(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.connect)

    def disconnect(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.disconnect)

    def echo(self, ebi: str) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.ebi_echo, {"ebi": ebi})

    def flash_ota(self, fname: str) -> dict[str, Any]:
        return self._cmd.flash(fname, cdm_bindings.flash_ebi_ota)

    def boot_ota(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.boot_ebi_ota)

    def mark_ota_as_ok(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.mark_ebi_ota_ok)

    def switch_to_factory(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.switch_ebi_factory)

    def switch_to_ota(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.switch_ebi_ota)

    def hard_reset(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.hard_reset, {"reset_erbi": True, "reset_bsdas": False})


class BsdasCmd:
    def __init__(self, cmd: CmdUtils) -> None:
        self._cmd: CmdUtils = cmd

    def echo(self, ebi: str, bsda: int) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.bsda_echo, {"ebi": ebi, "bsda": bsda})

    def boot_ota(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.boot_bsda_ota)

    def crash_system(self, idn: str) -> dict[str, Any]:
        """
        :param idn: A board identifier, it must be one of {R,L}{0,1,2}
        """
        check_valid_board_identifier(idn)
        return self._cmd.process(cdm_bindings.crash, {"board_identifier": idn})

    def get_coredump(self, idn: str, dpath: str) -> dict[str, Any]:
        """
        :param idn: A board identifier, it must be one of {R,L}{0,1,2}
        :param out_path: File where coredump will be saved
        """
        idn = check_valid_board_identifier(idn)
        # check that a BSDA is select and not only one of the ERBIs
        if len(idn) != 2:
            raise Exception("Invalid board identifier")

        return self._cmd.download(cdm_bindings.get_coredump, dpath, "coredump", {"board_identifier": idn})

    def prepare_ota(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.prepare_bsda_ota)

    def flash_ota(self, fname: str) -> dict[str, Any]:
        return self._cmd.flash(fname, cdm_bindings.flash_bsda_ota)

    def mark_ota_as_ok(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.mark_bsda_ota_ok)

    def switch_to_ota(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.switch_bsda_ota)

    def switch_to_factory(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.switch_bsda_factory)

    def hard_reset(self) -> dict[str, Any]:
        return self._cmd.process(cdm_bindings.hard_reset, {"reset_erbi": False, "reset_bsdas": True})



from pycdm import PyCDM, BsdaSettings, FramingType, EbiSettings

# PyCDM constructor accepts an optional argument as a string
# this argument is the IP of the CDM server.
# eg: cdm = PyCDM("127.0.0.1)
cdm = PyCDM("172.16.17.247")

cdm.ebi.connect()


bsda = BsdaSettings()
bsda.framing = FramingType.FULL
bsda.frame_duration_us = 1000
bsda.temperature_period_ms = 30000
bsda.adc_shadow = [0xf9f0, 0xf9f0]
bsda.log_uart = True
bsda.log_level = 3
bsda.raw_adc = 1
bsda.raw_channel = 1
bsda.disabled = False
bsda.temperature_enabled = True
bsda.light_enabled = True

bsda_disabled = BsdaSettings()
bsda_disabled.disabled = True

left = EbiSettings()
left.frame_reader_framing_type = FramingType.FULL
left.frame_reader_period_ms = 15
left.temperature_reader_period_ms = 30000
left.raw_board = 0
left.bsda_settings[0] = bsda_disabled
left.bsda_settings[1] = bsda_disabled
left.bsda_settings[2] = bsda_disabled
left.log_uart = True
left.log_level = 3

right = EbiSettings()
right.frame_reader_framing_type = FramingType.FULL
right.frame_reader_period_ms = 15
right.temperature_reader_period_ms = 30000
right.raw_board = 1
right.bsda_settings[0] = bsda
right.bsda_settings[1] = bsda
right.bsda_settings[2] = bsda
right.log_uart = True
right.log_level = 3

cdm.configure(left, right)
cdm.start_acq()
cdm.close()



import time
from pycdm import PyCDM
import cdm_bindings
from pycdm.structs import BsdaSettings, FramingType, EbiSettings
import zmq, json
import threading

bsda = BsdaSettings()
bsda.framing = FramingType.SUM_SQ
bsda.frame_duration_us = 1000
bsda.temperature_period_ms = 3000
bsda.voltage_period_ms = 5000
#bsda.adc_shadow = [0xf9f, 0xf9f]
bsda.adc_shadow = [0xf9f0, 0xf9f0]
bsda.log_uart = True
bsda.log_level = 2
bsda.raw_adc = 1
bsda.raw_channel = 1
bsda.disabled = False
bsda.imu_enabled = False
bsda.light_enabled = True
bsda.temperature_enabled = False
bsda.voltage_enabled = False

bsda2 = BsdaSettings()
bsda2.framing = FramingType.SUM_SQ
bsda2.frame_duration_us = 1000
bsda2.temperature_period_ms = 30000
bsda2.voltage_period_ms = 33000
#bsda2.adc_shadow = [0xf9f, 0xf9f]
bsda2.adc_shadow = [0xf9f0, 0xf9f0]
bsda2.log_uart = True
bsda2.log_level = 2
bsda2.raw_adc = 1
bsda2.raw_channel = 1
bsda2.disabled = False
bsda2.light_enabled = True
bsda2.temperature_enabled = True
bsda2.voltage_enabled = True

bsda_disabled = BsdaSettings()
bsda_disabled.disabled = True

left = EbiSettings()
left.frame_reader_framing_type = FramingType.SUM_SQ
left.frame_reader_period_ms = 15
left.temperature_reader_period_ms = 30000
left.voltage_reader_period_ms = 33000
left.raw_board = 0
left.bsda_settings[0] = bsda2
left.bsda_settings[1] = bsda2
left.bsda_settings[2] = bsda2
left.ntp_period_m = 15
left.log_uart = True
left.log_level = 2

right = EbiSettings()
right.frame_reader_framing_type = FramingType.SUM_SQ
right.frame_reader_period_ms = 15
right.temperature_reader_period_ms = 30000
right.voltage_reader_period_ms = 33000
right.raw_board = 1
right.bsda_settings[0] = bsda_disabled
right.bsda_settings[1] = bsda_disabled
right.bsda_settings[2] = bsda_disabled
right.log_uart = True
right.log_level = 2

cdm = PyCDM("127.0.0.1")
q = cdm.reg_msg("SYSTEM")

run = True
def trigger():
    while True:
        print(q.get_msg())
    #global run
    #run = False
        time.sleep(0.5)
        #print(cdm.bcg.get_info())
    #print(cdm.bcg.get_state())
    #print(cdm.bcg.get_info())


t = threading.Thread(target=trigger)
t.start()

print(cdm.ebi.connect())
print(cdm.bcg.get_info())
cdm.configure(left, right)
cdm.start_acq()
input("ASD")
a = 0
while run:
    cdm.start_acq()
    time.sleep(0.1)
    cdm.stop_acq()

print("Joinin")
t.join()
print("Closing")
cdm.close()
